import requests
import os


class Pdf:
    """ PDF generator lib"""

    def generate(self, url, filename="generated.pdf"):
        """ generate pdf from html url with optional filename"""
        json = {
            "output": "pdf",
            "url": url,
            "emulateScreenMedia": True,
            "ignoreHttpsErrors": True,
            "scrollPage": False,
            "pdf": {"width": "20.5cm", "height": "36.5cm"},
        }
        r = requests.get(
            url=f"{os.environ.get('SCREENSHOT_API', 'http://naas-screenshot:9000')}/api/render",
            json=json,
        )
        r.raise_for_status()
        open(filename, "wb").write(r.content)



from git import Repo, GitCommandError
import urllib.parse


class Git:
    def connect(self, config):
        """
        Description: This class accepst a JSON configuration and executes respective
        function as per the action specified in the configuration
        """

        self.config = dict(config)

        if [
            "username",
            "password",
            "github_url",
            "branch",
            "target_folder",
            "action",
            "commit_message",
        ] != [i for i in self.config.keys()]:
            print(
                "Configuration missing mandatory parameters (username,password,github_url,branch,target_folder,action)"
            )
            return

        if config.get("action", None) == "clone":
            self.clone(
                dest_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
                repourl=self.config.get("github_url", None),
                username=self.config.get("username", None),
                password=self.config.get("password", None),
            )
        elif config.get("action", None) == "commit":
            self.commit(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
                message=self.config.get("commit_message", None),
            )
        elif config.get("action", None) == "push":
            self.push(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
            )
        elif config.get("action", None) == "pull":
            self.pull(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
            )
        elif config.get("action", None) == "checkout":
            self.checkout(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
            )

    def clone(
        self,
        dest_dir="gitSource",
        branch="master",
        repourl=None,
        username=None,
        password=None,
    ):
        """ Clone the repo (dest_dir='gitSource', branch="master", repourl=None, username=None, password=None)"""
        try:
            if repourl[0:3] == "git":
                Repo.clone_from(repourl, dest_dir, branch=branch)
            else:
                urlsplit = repourl.split("//")
                authstring = username + ":" + urllib.parse.quote(password)
                urlsplit[1] = authstring + "@" + urlsplit[1]
                repourl = "//".join(urlsplit)
                Repo.clone_from(repourl, dest_dir, branch=branch)
            print("Clone successful")
        except GitCommandError as e:
            print("wrong command! \n", str(e))

    def commit(self, from_dir, branch, message="Auto commit"):
        """ Commit the files commit(from_dir,branch,message="Auto Commit Message")"""
        try:
            repo = Repo(from_dir)
            repo.git.execute(["git", "add", "--all"])
            repo.index.commit(message)
            repo.git.clean("-xdf")
            print("Commit successful!")
            return repo.remote()
        except GitCommandError as e:
            print("wrong command! \n", str(e))

    def push(self, from_dir, branch="master"):
        """ Push to the specified repository push(self,from_dir,branch="master")"""
        try:
            self.checkout(from_dir, branch)
            origin = self.commit(from_dir, branch)
            origin.push()
            print("Push successful")
        except GitCommandError as e:
            print("wrong command! \n", str(e))

    def pull(self, from_dir, branch="master"):
        """ Pull from the specified repository pull(from_dir,branch="master")"""
        try:
            self.checkout(from_dir, branch)
            origin = self.commit(from_dir, branch)
            origin.pull()
            print("Pull successful!")
        except GitCommandError as e:
            print("Wrong command! \n", str(e))

    def checkout(self, from_dir, branch="master"):
        """ Checkout to a spcified branch (from_dir, branch="master") """
        try:
            repo = Repo(from_dir)
            if repo.active_branch != branch:
                repo.git.clean("-xdf")
                repo.git.checkout(branch)
            print("Checkout successful")
        except GitCommandError as e:
            print("Wrong command! \n", str(e))



import requests


class Ifttt:
    def webhook(self, event, key, data=None):
        url = f"https://maker.ifttt.com/trigger/{event}/with/key/{key}"
        r = requests.post(
            url=url,
            json=data,
        )
        r.raise_for_status()
        return r



import requests


class Bubble:
    def run_workflow(self, url, token, data=None):
        headers = {"Authorization": f"Bearer {token}"}
        r = requests.post(
            headers=headers,
            url=url,
            json=data,
        )
        r.raise_for_status()
        r.json()



from airtable import Airtable as at


class Airtable:
    _airtable = None

    def connect(self, key, table):
        self._airtable = at(key, table)

    def get_all(self, *agrs):
        return self._airtable.get_all(*agrs)

    def insert(self, *agrs):
        return self._airtable.insert(*agrs)

    def search(self, *agrs):
        return self._airtable.search(*agrs)

    def update_by_field(self, *agrs):
        return self._airtable.update_by_field(*agrs)

    def delete_by_field(self, *agrs):
        return self._airtable.delete_by_field(*agrs)



import os
from urllib.parse import urljoin

import requests
import pandas as pd


BIG_NUM_TO_GETALL = 1000000


class Gsheet:
    def __init__(self):
        self.sheets_api = os.getenv("GSHEETS_API")
        if not self.sheets_api:
            raise ValueError("GSHEETS_API not defined!")

    def get(
        self,
        spreadsheet_id: str,
        sheet_name: str,
        items_per_page: int = BIG_NUM_TO_GETALL,
    ) -> pd.DataFrame:
        resp = requests.get(
            urljoin(self.sheets_api, f"{spreadsheet_id}/{sheet_name}"),
            params={"perPage": items_per_page},
        )
        if resp.status_code != 200:
            raise ValueError(resp.text)
        data = resp.json()
        df = pd.DataFrame(data=data["data"], columns=data["columns"])
        return df



from .ftpbase import Ftpbase
from .ftpsbase import Ftpsbase


class Ftp:
    """FTP subclass"""

    _ftp = None

    def connect(
        self,
        user,
        passwd,
        host="ftps.toucantoco.com",
        port=990,
        secure=False,
        force_prot=False,
    ):
        if secure:
            self._ftp = Ftpsbase()
        else:
            self._ftp = Ftpbase()
        self._ftp.connect(host=host, port=port)
        self._ftp.login(user=user, passwd=passwd)
        self.__force_prot = force_prot

    def __before_all(self):
        if self.__force_prot:
            self._ftp.prot_c()
            self._ftp.prot_p()

    def get_file(self, path):
        self.__before_all()
        return self._ftp.get_file(path)

    def send_file(self, path, dest_path=None):
        self.__before_all()
        self._ftp.send_file(path, dest_path)

    def list_directory(self, dirName):
        self.__before_all()
        return self._ftp.list_directory(dirName)

    def help(self):
        print(f"=== {type(self).__name__} === \n")
        print(".get_file(path) => get file from ftp path\n")
        print(".send_file(path, dest_path) => send file to ftp path\n")
        print(".list_directory(path) => do ls in ftp in path\n")



from .integromat import Integromat
from .darkknight import DarkKnight
from .ftp import Ftp
from .geolocator import GeoLocator
from .git import Git
from .gsheet import Gsheet
from .healthcheck import HealthCheck
from .email import Email
from .mongo import Mongo
from .pdf import Pdf
from .plotly import Plotly
from .html import Html
from .cityfalcon import Cityfalcon
from .prediction import Prediction
from .sentiment_analysis import SentimentAnalysis
from .toucan import Toucan
from .yahoo import Yahoo
from .airtable import Airtable
from .zappier import Zappier
from .ifttt import Ifttt
import requests

__version__ = "0.18.5"

__github_repo = "jupyter-naas/drivers"

darkknight = DarkKnight()
airtable = Airtable()
zappier = Zappier()
integromat = Integromat()
ifttt = Ifttt()
yahoo = Yahoo()
ftp = Ftp()
git = Git()
geoLocator = GeoLocator()
gsheet = Gsheet()
health_check = HealthCheck()
email = Email()
mongo = Mongo()
pdf = Pdf()
plotly = Plotly()
html = Html()
cityfalcon = Cityfalcon()
prediction = Prediction()
sentiment_analysis = SentimentAnalysis()
toucan = Toucan()


def version():
    print(__version__)


def get_last_version():
    url = f"https://api.github.com/repos/{__github_repo}/tags"
    response = requests.get(url, headers={"Accept": "application/vnd.github.v3+json"})
    return response.json()[0]["name"]


def up_to_date():
    return get_last_version() == version()
